﻿#region FILE DESCRIPTION
//-----------------------------------------------------------------------------
// MuScreenManager
// 
// MenuItem.cs
//-----------------------------------------------------------------------------
#endregion

#region USING
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
#endregion USING

namespace MuScreenManager
{
    /// <summary>
    /// Item mainly used for a list style menu setup - specifically for MenuScreen
    /// </summary>
    public class MenuItem
    {
                                                #region PROPERTIES

        /// <summary>
        /// Current ScreenManager controlling this menu and thus MenuItem
        /// </summary>
        private ScreenManager screenManager;

        /// <summary>
        /// Font used to draw menu items on screen
        /// </summary>
        private SpriteFont font;

        /// <summary>
        /// Text to display for this button
        /// </summary>
        private string text;

        /// <summary>
        /// Position of the text on screen
        /// </summary>
        private Vector2 position;

        /// <summary>
        /// Origin of that text (center)
        /// </summary>
        private Vector2 stringOrigin;

        /// <summary>
        /// Return the hitbox that the menuItem is occupying on screen
        /// </summary>
        private Rectangle hitbox;
        public Rectangle Hitbox
        {
            get { return hitbox; }
        }

                                                #endregion PROPERTIES

                                                #region EVENTS

        /// <summary>
        /// Event for use on button hit
        /// </summary>
        public event EventHandler<MenuItemEventArgs> OnButtonSelect;

        /// <summary>
        /// Method used to execute the OnButtonSelect event
        /// </summary>
        public void RunOnButtonSelect()
        {
            if (OnButtonSelect != null)
                OnButtonSelect(this, new MenuItemEventArgs());
        }

                                                #endregion EVENTS

                                                #region METHODS

        /// <summary>
        /// Initialize the MenuItem components of this object
        /// </summary>
        /// <param name="text">Text to display for this button</param>
        /// <param name="position">Position of the button on screen</param>
        /// <param name="screenManager">Current ScreenManager controlling this menu</param>
        public MenuItem(string text, Vector2 position, ScreenManager screenManager)
        {
            //Store the ScreenManager immediately
            this.screenManager = screenManager;

            //Check if there is a general font in ScreenManager
            if (screenManager.Font != null)
            {
                //No errors, store all required information
                this.text = text;
                this.position = position;
                this.font = screenManager.Font;
            }
            //No font loaded in ScreenManager, let peoples know
            else
                throw new ContentLoadException("Font not loaded in ScreenManager");

            //Measure the string and set up the hitbox for the menuitem on screen
            Vector2 size = font.MeasureString(text);
            hitbox = new Rectangle((int)(position.X - (size.X / 2)), (int)(position.Y - (size.Y / 2)), (int)size.X, (int)size.Y);

            //Calculate the string origin
            this.stringOrigin = size / 2;
        }

        /// <summary>
        /// Initialize the MenuItem components of this object
        /// </summary>
        /// <param name="text">Text to display for this button</param>
        /// <param name="position">Position of the button on screen</param>
        /// <param name="font">Font used to draw menu items on screen</param>
        public MenuItem(string text, Vector2 position, SpriteFont font)
        {
            //No error check required, store all required information
            this.text = text;
            this.position = position;
            this.font = font;

            //Measure the string and set up the hitbox for the menuitem on screen
            Vector2 size = font.MeasureString(text);
            hitbox = new Rectangle((int)(position.X - (size.X / 2)), (int)(position.Y - (size.Y / 2)), (int)size.X, (int)size.Y);

            //Calculate the string origin
            this.stringOrigin = size / 2;
        }

        /// <summary>
        /// Draw this menu item on screen following parameters
        /// </summary>
        /// <param name="spriteBatch">SpriteBatch used to draw these objects on screen</param>
        /// <param name="gameTime">Provides a snapshot of the game time</param>
        /// <param name="isSelected">Choose if this menu item is already selected</param>
        public void Draw(SpriteBatch spriteBatch, GameTime gameTime, bool isSelected)
        {
            if (isSelected)
            {
                //Make the menu item pulse. Calculate a rate of pulsation using Sin
                float pulse = (float)((Math.Sin(gameTime.TotalGameTime.TotalSeconds * 4) * 0.15) + 0.15);
                spriteBatch.DrawString(font, text, position, Color.Yellow, 0, stringOrigin, 1 + pulse, SpriteEffects.None, 0);
            }
            else
                spriteBatch.DrawString(font, text, position, Color.White, 0, stringOrigin, 1, SpriteEffects.None, 0);
        }

                                                #endregion METHODS
    }

    /// <summary>
    /// Placeholder - Used to pass data from this to event method - currently unneeded but good just in case
    /// </summary>
    public class MenuItemEventArgs : EventArgs
    { }
}
